@@ -21,13 +21,12 @@
  * Carsten Haitzler <raster@rasterman.com>
  */
 
+#include <stdio.h>
 #include <string.h>
 
 #include "svg.h"
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#include <rsvg.h>
 
-GCache *pixbuf_cache = NULL;
+static GCache *pixbuf_cache = NULL;
 
 static GdkPixbuf *
 bilinear_gradient (GdkPixbuf    *src,
@@ -413,7 +412,7 @@ pixbuf_render (GdkPixbuf    *src,
 			x_scale, y_scale,
 			GDK_INTERP_BILINEAR);
 
-      gdk_pixbuf_unref (partial_src);
+      g_object_unref (partial_src);
 
       x_offset = 0;
       y_offset = 0;
@@ -435,7 +434,7 @@ pixbuf_render (GdkPixbuf    *src,
 				       GDK_PIXBUF_ALPHA_FULL, 128,
 				       GDK_RGB_DITHER_NORMAL,
 				       0, 0);
-  gdk_pixbuf_unref (tmp_pixbuf);
+  g_object_unref (tmp_pixbuf);
 }
 
 ThemePixbuf *
@@ -443,7 +442,7 @@ theme_pixbuf_new (void)
 {
   ThemePixbuf *result = g_new0 (ThemePixbuf, 1);
   result->filename = NULL;
-  result->pixbuf = NULL;
+  result->handle = NULL;
 
   result->stretch = TRUE;
   result->border_left = 0;
@@ -465,10 +464,10 @@ void
 theme_pixbuf_set_filename (ThemePixbuf *theme_pb,
 			   const char  *filename)
 {
-  if (theme_pb->pixbuf)
+  if (theme_pb->handle)
     {
-      g_cache_remove (pixbuf_cache, theme_pb->pixbuf);
-      theme_pb->pixbuf = NULL;
+      g_cache_remove (pixbuf_cache, theme_pb->handle);
+      theme_pb->handle = NULL;
     }
 
   if (theme_pb->filename)
@@ -546,11 +545,11 @@ compute_hint (GdkPixbuf *pixbuf,
 }
 
 static void
-theme_pixbuf_compute_hints (ThemePixbuf *theme_pb)
+theme_pixbuf_compute_hints (ThemePixbuf *theme_pb, GdkPixbuf *pixbuf)
 {
   int i, j;
-  gint width = gdk_pixbuf_get_width (theme_pb->pixbuf);
-  gint height = gdk_pixbuf_get_height (theme_pb->pixbuf);
+  gint width = gdk_pixbuf_get_width (pixbuf);
+  gint height = gdk_pixbuf_get_height (pixbuf);
 
   if (theme_pb->border_left + theme_pb->border_right > width ||
       theme_pb->border_top + theme_pb->border_bottom > height)
@@ -610,7 +609,7 @@ theme_pixbuf_compute_hints (ThemePixbuf *theme_pb)
 	      break;
 	    }
 
-	  theme_pb->hints[i][j] = compute_hint (theme_pb->pixbuf, x0, x1, y0, y1);
+	  theme_pb->hints[i][j] = compute_hint (pixbuf, x0, x1, y0, y1);
 	}
     }
   
@@ -627,9 +626,6 @@ theme_pixbuf_set_border (ThemePixbuf *theme_pb,
   theme_pb->border_right = right;
   theme_pb->border_top = top;
   theme_pb->border_bottom = bottom;
-
-  if (theme_pb->pixbuf)
-    theme_pixbuf_compute_hints (theme_pb);
 }
 
 void
@@ -637,46 +633,105 @@ theme_pixbuf_set_stretch (ThemePixbuf *theme_pb,
 			  gboolean     stretch)
 {
   theme_pb->stretch = stretch;
-
-  if (theme_pb->pixbuf)
-    theme_pixbuf_compute_hints (theme_pb);
 }
 
-static GdkPixbuf *
-pixbuf_cache_value_new (gchar *filename)
+static void
+svg_cache_value_free(gpointer foo)
 {
-  GError *err = NULL;
-    
-  GdkPixbuf *result = rsvg_pixbuf_from_file (filename, &err);
-  if (!result)
+  RsvgHandle * handle;
+
+  handle = (RsvgHandle *)foo;
+  if(handle != NULL)
+    g_object_unref(G_OBJECT(handle));
+}
+
+#define SVG_BUFFER_SIZE (1024*8)
+
+static RsvgHandle *
+svg_cache_value_new (gchar *filename)
+{
+  RsvgHandle *result = NULL;
+  FILE *fp;
+
+  fp = fopen(filename, "rb");
+  if(fp)
     {
-      g_warning ("Rsvg theme: Cannot load SVG file %s: %s\n",
-		 filename, err->message);
-      g_error_free (err);
+      size_t nread;
+      unsigned char buf[SVG_BUFFER_SIZE];
+
+      result = rsvg_handle_new();
+      while((nread = fread(buf, 1, sizeof(buf), fp)) > 0)
+	rsvg_handle_write(result, buf, nread, NULL);
+
+      fclose(fp);
+      rsvg_handle_close(result, NULL);
+    }
+  else
+    {
+      g_warning("Couldn't load theme part: %s\n", filename);
     }
 
   return result;
 }
 
-GdkPixbuf *
-theme_pixbuf_get_pixbuf (ThemePixbuf *theme_pb)
+struct SizeInfo
 {
-  if (!theme_pb->pixbuf)
+  gint width, height;
+};
+
+static void set_size_fn(gint *width, gint *height, gpointer foo)
+{
+  struct SizeInfo * info = (struct SizeInfo *)foo;
+
+  *width = info->width;
+  *height = info->height;
+}
+
+static GdkPixbuf *
+get_pixbuf(RsvgHandle * handle, gint width, gint height)
+{
+  GdkPixbuf * result;
+
+  if(!handle)
+    return NULL;
+
+  if(width > 0 && height > 0)
+    {
+      struct SizeInfo info;
+
+      info.width = width;
+      info.height = height;
+      
+      rsvg_handle_set_size_callback(handle, set_size_fn, &info, NULL);
+    }
+
+  result = rsvg_handle_get_pixbuf(handle);
+
+  return result;
+}
+
+GdkPixbuf *
+theme_pixbuf_get_pixbuf (ThemePixbuf *theme_pb, gint width, gint height)
+{
+  GdkPixbuf *result = NULL;
+
+  if (!theme_pb->handle)
     {
       if (!pixbuf_cache)
-	pixbuf_cache = g_cache_new ((GCacheNewFunc)pixbuf_cache_value_new,
-				    (GCacheDestroyFunc)gdk_pixbuf_unref,
+	pixbuf_cache = g_cache_new ((GCacheNewFunc)svg_cache_value_new,
+				    (GCacheDestroyFunc)svg_cache_value_free,
 				    (GCacheDupFunc)g_strdup,
 				    (GCacheDestroyFunc)g_free,
 				    g_str_hash, g_direct_hash, g_str_equal);
       
-      theme_pb->pixbuf = g_cache_insert (pixbuf_cache, theme_pb->filename);
-
-      if (theme_pb->stretch)
-	theme_pixbuf_compute_hints (theme_pb);
+      theme_pb->handle = g_cache_insert (pixbuf_cache, theme_pb->filename);
     }
   
-  return theme_pb->pixbuf;
+  result = get_pixbuf(theme_pb->handle, width, height);
+  if(result)
+    theme_pixbuf_compute_hints(theme_pb, result);
+  
+  return result;
 }
 
 void
@@ -691,7 +746,7 @@ theme_pixbuf_render (ThemePixbuf  *theme_pb,
 		     gint          width,
 		     gint          height)
 {
-  GdkPixbuf *pixbuf = theme_pixbuf_get_pixbuf (theme_pb);
+  GdkPixbuf *pixbuf = theme_pixbuf_get_pixbuf (theme_pb, width, height);
   gint src_x[4], src_y[4], dest_x[4], dest_y[4];
   gint pixbuf_width = gdk_pixbuf_get_width (pixbuf);
   gint pixbuf_height = gdk_pixbuf_get_height (pixbuf);
@@ -804,4 +859,6 @@ theme_pixbuf_render (ThemePixbuf  *theme_pb,
 	  gdk_pixmap_unref (tmp_pixmap);
 	}
     }
+
+  g_object_unref(G_OBJECT(pixbuf));
 }
