@@ -32,7 +32,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
+#endif
 #include <errno.h>
 #include <math.h>
 
@@ -250,6 +252,25 @@ _rsvg_css_hand_normalize_length (const RsvgLength * in, gdouble pixels_per_inch,
     return 0;
 }
 
+double
+_rsvg_css_hand_normalize_length_sub (const RsvgLength * in, gdouble length, 
+				     gdouble pixels_per_inch, gdouble width_or_height, 
+				     gdouble font_size)
+{
+    if (in->factor == '\0')
+        return length;
+    else if (in->factor == 'p')
+        return length * width_or_height;
+    else if (in->factor == 'm')
+        return length * font_size;
+    else if (in->factor == 'x')
+        return length * font_size / 2.;
+    else if (in->factor == 'i')
+        return length * pixels_per_inch;
+
+    return 0;
+}
+
 gboolean
 rsvg_css_param_match (const char *str, const char *param_name)
 {
