@@ -50,8 +50,25 @@ rsvg_node_draw (RsvgNode * self, RsvgDrawingCtx * ctx, int dominate)
     if (!state->visible)
         return;
 
+    if (g_slist_find(ctx->ptrs, self) != NULL)
+    {
+        /*
+         * 5.3.1 of the SVG 1.1 spec (http://www.w3.org/TR/SVG11/struct.html#HeadOverview)
+         * seems to indicate ("URI references that directly or indirectly reference
+         * themselves are treated as invalid circular references") that circular
+         * references are invalid, and so we can drop them to avoid infinite recursion.
+         * 
+         * See also http://bugzilla.gnome.org/show_bug.cgi?id=518640
+         */
+        g_warning("Circular SVG reference noticed, dropping");
+        return;
+    }
+    ctx->ptrs = g_slist_append(ctx->ptrs, self);
+
     self->draw (self, ctx, dominate);
     ctx->drawsub_stack = stacksave;
+
+    ctx->ptrs = g_slist_remove(ctx->ptrs, self);
 }
 
 /* generic function for drawing all of the children of a particular node */
@@ -328,9 +345,16 @@ rsvg_node_svg_set_atts (RsvgNode * self, RsvgHandle * ctx, RsvgPropertyBag * att
             svg->preserve_aspect_ratio = rsvg_css_parse_aspect_ratio (value);
         if ((value = rsvg_property_bag_lookup (atts, "width")))
             svg->w = _rsvg_css_parse_length (value);
+	else {
+	    svg->w.length = -1;
+	    svg->w.factor = '\0';
+	}
         if ((value = rsvg_property_bag_lookup (atts, "height")))
             svg->h = _rsvg_css_parse_length (value);
-
+	else {
+	    svg->h.length = -1;
+	    svg->h.factor = '\0';
+	}
 		/* 
 		 * x & y attributes have no effect on outermost svg
 		 * http://www.w3.org/TR/SVG/struct.html#SVGElement 
