@@ -24,6 +24,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <locale.h>
 
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
@@ -309,7 +310,7 @@ save_pixbuf (GObject * ignored, gpointer user_data)
                 errmsg = gtk_message_dialog_new (GTK_WINDOW (info->window),
                                                  GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                                  GTK_MESSAGE_WARNING,
-                                                 GTK_BUTTONS_CLOSE, err->message);
+                                                 GTK_BUTTONS_CLOSE, "%s", err->message);
 
                 gtk_dialog_run (GTK_DIALOG (errmsg));
 
@@ -536,8 +537,6 @@ populate_window (GtkWidget * win, ViewerCbInfo * info, int xid, gint win_width,
 
 		gtk_widget_size_request(toolbar, &requisition);
 
-		g_print("w: %d | h: %d\n", requisition.width, requisition.height);
-
 		/* HACK: adjust for frame width & height + packing borders */
 		img_height += requisition.height + 30;
 		win_height += requisition.height + 30;
@@ -694,6 +693,9 @@ main (int argc, char **argv)
         {NULL}
     };
 
+	/* Set the locale so that UTF-8 filenames work */
+	setlocale(LC_ALL, "");
+
     g_thread_init (NULL);
 
     info.pixbuf = NULL;
